﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;

namespace ThinkGeoSaveShapeDbfColumnsProblem
{
    class ShapeFiles
    {
        public static void SaveShapeFile(FeatureLayer fl, string shapeName, ManagedProj4Projection proj4)
        {

            // Get the columns 
            Collection<DbfColumn> allColumns = new Collection<DbfColumn>();

            fl.Open();
            foreach (FeatureSourceColumn column in fl.FeatureSource.GetColumns())
            {

                if (column.ColumnName == "Shape" || column.ColumnName == "SHAPE") continue;

                // Now the FeatureSourceColumn has a type, so cast it to a DbfColumnType
                DbfColumnType colType = (DbfColumnType)Enum.Parse(typeof(DbfColumnType), column.TypeName.ToString());

                // Need to make the column maximum a high number as we don't know what it could be
                allColumns.Add(new DbfColumn(column.ColumnName, colType, 500, 0));

            }


            // Get they type of the shape file and output it.  I.e. it may be a Polygon or Points or Area.
            WellKnownType shapeType = fl.FeatureSource.GetFirstFeaturesWellKnownType();
            fl.Close();
            ShapeFileType shapeFileType = new ShapeFileType();

            switch (shapeType)
            {
                case WellKnownType.Polygon:
                    {
                        shapeFileType = ShapeFileType.Polygon;
                        break;
                    }
                case WellKnownType.Line:
                    {
                        shapeFileType = ShapeFileType.Polyline;
                        break;
                    }
                case WellKnownType.Multiline:
                    {
                        shapeFileType = ShapeFileType.Polyline;
                        break;
                    }
                case WellKnownType.Multipoint:
                    {
                        shapeFileType = ShapeFileType.Multipoint;
                        break;
                    }
                case WellKnownType.Point:
                    {
                        shapeFileType = ShapeFileType.Point;
                        break;
                    }
                case WellKnownType.Multipolygon:
                    {
                        shapeFileType = ShapeFileType.Polygon;
                        break;
                    }
            }

            ShapeFileFeatureLayer.CreateShapeFile(shapeFileType,
                shapeName, allColumns, Encoding.ASCII, OverwriteMode.Overwrite);

            ShapeFileFeatureLayer shapeFileLayer = new ShapeFileFeatureLayer(
                shapeName, ShapeFileReadWriteMode.ReadWrite);

            fl.Open();
            proj4.Open();
            foreach (Feature feature in fl.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns))
            {
                shapeFileLayer.Open();
                try
                {
                    Feature convertedFeature = proj4.ConvertToInternalProjection(feature);

                    shapeFileLayer.EditTools.BeginTransaction();
                    shapeFileLayer.EditTools.Add(convertedFeature);
                    TransactionResult result = shapeFileLayer.EditTools.CommitTransaction();
                }
                finally
                {
                    shapeFileLayer.Close();
                }
            }
            fl.Close();

            // Output a prj file as well.
            string prj = ManagedProj4Projection.ConvertProj4ToPrj(proj4.InternalProjectionParametersString);
            string fileName = shapeName.Substring(0, shapeName.Length - 4) + ".prj";
            string folderName = new FileInfo(shapeName).DirectoryName;

            StreamWriter streamFile = new StreamWriter(fileName, false);
            streamFile.WriteLine(prj);
            streamFile.Close();

            string outMessage = string.Format("Output {0}", new FileInfo(shapeName).Name);

            MessageBox.Show(outMessage, "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);

        }

    }
}
